/**
 * @author http://kirill-novitchenko.com
 */

var TB_version = '2.0.0',	// Plugin version 
TB_ws,						// path to web service
TB_config,					// configuration options
TB_minTweetID,				// ID of the oldest tweet shown
TB_maxTweetID,				// ID of the newest tweet shown
TB_tweetsShown = 0,			// number of tweets shown
TB_rateLimitData,
TB_mode = 'widget';			// widget or archive

function TB_init() {
	// create web services endpoint
	TB_ws = TB_pluginPath + '/ws.php';
	
	// show loading indicator
	TB_showLoader();
	
	// check mode
	if(jQuery('#tweetblender').hasClass('archive')) {
		TB_mode = 'archive';
	}
	
	// add action to twitter logo
	jQuery('#twitterlogo').click(function(){
		var sourcesHTML = 'Powered by Tweet Blender plugin v' + TB_version + ' blending ';
		jQuery.each(TB_config.general_source_ids.split(','),function(i,src) {
			sourcesHTML += '<a target="_blank" href="';
			if (src.charAt(0) == '#') {
			 	sourcesHTML += 'http://search.twitter.com/search?' + escape(src);
			}
			else {
			 	sourcesHTML += 'http://twitter.com/' + src;
			}
			sourcesHTML += '">' + src + '</a> ';
		});		
		TB_showMessage(sourcesHTML);
	});

	// get config options and blend
	jQuery.ajax({
		url: TB_ws + '?action=get_config',
		dataType: 'json',
		success: function(json){
			if (typeof(json.general_source_ids) == 'undefined') {
				TB_showMessage('Twitter sources to blend are not defined');
			}
			else {
				TB_config = json;
				TB_blend();
			}
		},
		error: TB_ajaxError
	});
}

function TB_blend() {
	// iterate over sources
	var sources = TB_config.general_source_ids.split(',');
	
	jQuery.each(sources,function(i,src) {
		TB_getTweets(src);
	});		
}
	
function TB_getTweets(source){
	// check limit for user viwing the page
	jQuery.ajax({
		url: 'http://twitter.com/account/rate_limit_status.json',
		dataType: 'jsonp',
		success: function(json){
			if (json.remaining_hits == 0) {
				TB_getCachedTweets(source);
			}
			// else, get new feed
			else {
				TB_getFreshTweets(source);
			}
		},
		error: function(){
			TB_getCachedTweets(source);
		}
	});
}

function TB_getCachedTweets(source) {
	jQuery.ajax({
		url: TB_pluginPath + '/cache/' + escape(source) + '.json',
		dataType: 'json',
		success: TB_addTweets,
		error: function(){
			TB_showMessage("Can't get tweets for " + source + ". You've reached your maximum Twitter connection limit and cache is not available.")
		}
	});	
}

function TB_getFreshTweets(source) {
	// if it's a hashtag - use search API
	if(source.charAt(0) == '#') {
		jQuery.ajax({
			dataType: 'jsonp',
			url: 'http://search.twitter.com/search.json',
			data: ({
				q: source,
				rpp: TB_config.archive_tweets_num,
				page: 1
			}),
			success: function (json) {
				TB_cacheData(source,json);
				TB_addTweets(json);
			},
			error: function(){
				TB_getCachedTweets(source);
			}
		});
	}
	// if it's a screen name - use REST API
	else {
		jQuery.ajax({
			url: 'http://twitter.com/statuses/user_timeline.json',
			dataType: 'jsonp',
			data: ({
				screen_name: source,
				count: TB_config.archive_tweets_num
			}),
			success: function (json) {
				TB_cacheData(source,json);
				TB_addTweets(json);
			},
			error: function() {
				TB_getCachedTweets(source);
			}
		});
	}
}

function TB_cacheData(source,jsonData) {
	jQuery.ajax({
		url: 		TB_ws + '?action=cache_data',
		type:		'POST',
		dataType: 	'json',
		data: ({
			source: source,
			data: jQuery.toJSON(jsonData)
		})
	});
}
	
function TB_addTweets(jsonData) {
	
	// hide loader and show tweet list
	TB_showTweetList();

	var tweets = jsonData;
	if (typeof(jsonData.results) != 'undefined') {
		tweets = jsonData.results;
	}
	
	jQuery.each(tweets,function(i,tweet) {
		// if this tweet already in the set, skip it
		if (jQuery('#' + tweet.id).length > 0) {
			return true;
		}
		// if this is the first tweet, just add it and set it to be both min and max
		else if (TB_tweetsShown == 0) {
			TB_minTweetID = tweet.id;
			TB_maxTweetID = tweet.id;			

			// add at the end
			jQuery('#tweetlist').append(TB_makeHTML(tweet));
			TB_tweetsShown++;
		}
		// if tweet older than the oldest
		else if (TB_minTweetID > 0 && tweet.id < TB_minTweetID) {
			// if we are at max already, no need to work through the rest of this set as the rest will be older
			if (TB_tweetsShown == TB_config[TB_mode+'_tweets_num']) {
				return false;
			}
			else {
				// add at the end
				jQuery('#tweetlist').append(TB_makeHTML(tweet));
				TB_tweetsShown++;

				// make it the oldest
				TB_minTweetID = tweet.id;
			}
		}
		// if tweet is newer than the newest
		else if (TB_maxTweetID > 0 && tweet.id > TB_maxTweetID) {

			// if we are at max already, remove bottom tweet
			TB_enforceLimit();
			
			// add in the beginning
			jQuery('#tweetlist').prepend(TB_makeHTML(tweet));
			TB_tweetsShown++;

			// make it the newest
			TB_maxTweetID = tweet.id;
		}
		// if tweet is in the middle
		else {

			// if we are at max already, remove bottom tweet
			TB_enforceLimit();

			// traverse currently shown tweets and insert in the appropriate spot
			var prevTweetID = TB_maxTweetID;
			jQuery('#tweetlist li').each(function(i,nextTweet){
				if (tweet.id < prevTweetID && tweet.id > nextTweet.id) {
					jQuery('#'+prevTweetID).after(TB_makeHTML(tweet));
					TB_tweetsShown++;
					return false;
				}
				prevTweetID = nextTweet.id;
			});
		}
		
	});

	if (tweets.length > 0 && TB_mode != 'archive') {
		jQuery('#archivelink').show();
	}
}


function TB_enforceLimit() {
	if (TB_tweetsShown == TB_config[TB_mode+'_tweets_num']) {
		var lastTweet = jQuery('#' + TB_minTweetID),
		nextToLastTweet = lastTweet.prev('li');
		
		// remove last tweet
		lastTweet.remove();
		TB_tweetsShown--;
		
		// make next to last to be last now
		TB_minTweetID = parseInt(nextToLastTweet.attr('id'));
	}
}

function TB_makeHTML(tweet) {
		
	var tweetHTML = '',
	tweetDate = new Date();
	
	// add screen name if from_user is given
	if (typeof(tweet.user) == 'undefined' && tweet.from_user) {
		tweet.user = {
			screen_name: tweet.from_user
		};
	}
	
	tweetHTML += '<li id="' + tweet.id + '">';
	
	// show photo if requested
	if (TB_config[TB_mode+'_show_photos']) {

		// add image url
		if (!tweet.user.profile_image_url && tweet.profile_image_url) {
			tweet.user.profile_image_url = tweet.profile_image_url;
		}

		tweetHTML += '<a class="tb_photo" href="http://twitter.com/' + tweet.user.screen_name + '" target="_blank">';
		tweetHTML += '<img src="' + tweet.user.profile_image_url + '" alt="' + tweet.user.screen_name + '"/>';
		tweetHTML += '</a>';
	}

	// show author
	tweetHTML += '<span class="tb_author"><a href="http://twitter.com/' + tweet.user.screen_name + '" target="_blank">' + tweet.user.screen_name + '</a>: </span> ';

	// show text
	if (TB_config.general_link_urls) {
		tweet.text = tweet.text.replace(/(https?:\/\/\S+)/gi,'<a href="$1" target="_blank">$1</a>'); 
	}
	if (TB_config.general_link_screen_names) {
		tweet.text = tweet.text.replace(/\@([^\s\.\:]+)/gi,'<a href="http://twitter.com/$1" target="_blank">@$1</a>'); 
	}
	if (TB_config.general_link_hash_tags) {
		tweet.text = tweet.text.replace(/\#([^\s\,]+)/gi,'<a href="http://search.twitter.com/search?q=%23$1" target="_blank">#$1</a>'); 
	}
	tweetHTML += '<span class="tb_msg">' + tweet.text + '</span><br/>';

	// start tweet footer with info
	tweetHTML += ' <span class="tb_tweetinfo">';
	
	// show timestamp
	tweetHTML += '<a href="http://twitter.com/' + tweet.user.screen_name + '/statuses/' + tweet.id + '">';
	tweetDate.setTime(Date.parse(tweet.created_at));
	if (TB_config.general_timestamp_format) {
		tweetHTML += jQuery.PHPDate(TB_config.general_timestamp_format,tweetDate);
	}
	else {
		tweetHTML += jQuery.timeago(tweetDate);
	} 
	tweetHTML += '</a>';
	
	// show source if requested
	if (TB_config[TB_mode+'_show_source'] && tweet.source) {
		tweetHTML += ' from ' + jQuery('<textarea/>').html(tweet.source).val();
	}
	
	// end tweet footer
	tweetHTML += '</span>';
		
	// end tweet	
	tweetHTML += "</li>\n";

	return tweetHTML;
}


function TB_showLoader() {
	jQuery('#refreshlink img').attr('src',TB_pluginPath + '/img/ajax-refresh.gif');
	jQuery('#refreshlink').addClass('loading');
}

function TB_hideLoader() {
	jQuery('#tb_loading').hide();
	jQuery('#refreshlink img').attr('src',TB_pluginPath + '/img/ajax-refresh-icon.gif');
	jQuery('#refreshlink').removeClass('loading');
}

function TB_showTweetList() {
	TB_hideLoader();
	jQuery('#tweetlist').show();
}

function TB_showMessage(msg) {
	TB_hideLoader();
	jQuery('#tweetlist').before('<div class="tb_msg">' + msg + '</div>');
	jQuery('div.tb_msg').slideDown();
	setTimeout('TB_hideMessage()',10000);
}

function TB_hideMessage() {
	jQuery('div.tb_msg').slideUp();
}

function TB_ajaxError(XMLHttpRequest, textStatus, errorThrown) {
	TB_showMessage('Error: ' + textStatus);
}

jQuery(document).ready(function() {
	TB_init();
});


