<?php
/*
Plugin Name: Tweet Blender
Plugin URI: http://kirill-novitchenko.com/tweet-blender/
Description: Provides sidebar widget and archive page that shows tweets frrom multiple authors or hashtags blended together.
Version: 2.0.0
Author: Kirill Novitchenko
Author URI: http://kirill-novitchenko.com
*/

/*  Copyright 2009  Kirill Novitchenko  (email : knovitchenko@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// register settings page
add_action('admin_menu', 'tweet_blender_admin_menu');
function tweet_blender_admin_menu() {
  add_options_page('Tweet Blender Settings', 'Tweet Blender', 8, __FILE__, 'tweet_blender_admin_page');
}

// register stylesheet, javascript & jQuery
add_action('wp_head', 'add_header_code');
function add_header_code() {
	echo '<link type="text/css" rel="stylesheet" href="' . plugins_url('tweet-blender/css/tweets.css') . '" />' . "\n";
	echo '<script type="text/javascript">var TB_pluginPath = "' . plugins_url('/tweet-blender') . '";</script>';
}

// add javascript with dependency on jQuery to public pages only
add_action("template_redirect","tb_load_js");
function tb_load_js() {
	wp_enqueue_script('phpdate', '/' . PLUGINDIR . '/tweet-blender/js/jquery.phpdate.js', array('jquery'));
	wp_enqueue_script('timeago', '/' . PLUGINDIR . '/tweet-blender/js/jquery.timeago.js', array('jquery'));
	wp_enqueue_script('json', '/' . PLUGINDIR . '/tweet-blender/js/jquery.json.min.js', array('jquery'));
	wp_enqueue_script('tbmain', '/' . PLUGINDIR . '/tweet-blender/js/main.min.js', array('jquery'));
}

// register the widget and its control panel
add_action("widgets_init", "tb_init_widget");
function tb_init_widget(){
	register_sidebar_widget("Tweet Blender", "tb_widget");
	register_widget_control('Tweet Blender', 'tb_widget_control_panel');
}

// hookup filter to add tweet list to the content of archive page
add_filter('the_content', 'tb_add_archive_page_content');
function tb_add_archive_page_content($content = '') {
	global $post;
	
	// work with pages only, ignore blog posts
	if ($post->post_type != 'page') {
		return $content;
	}
	
	// if looking at archive page, apend list of tweets to content
	if ($post->ID == tb_get_archive_post_id()) {
		$archive_html = '<div id="tweetblender" class="archive">';
		$archive_html .= '<img id="twitterlogo" src="' . plugins_url('tweet-blender/img/twitter-logo.png') . '" alt="Twitter Logo">';
		$archive_html .= '<a id="infolink" href="http://kirill-novitchenko.com/tweet-blender/" target="_blank" title="TweetBlender info"> </a>';
		$archive_html .= '<a id="refreshlink" href="javascript:TB_showLoader();TB_blend();" title="Refresh Tweets"><img src="' . plugins_url('tweet-blender/img/ajax-refresh-icon.gif') . '" alt="Refresh"></a>';
		$archive_html .= '<div id="tb_loading">Loading...</div>';
		$archive_html .= '<ul id="tweetlist" style="display:none"></ul>';
		$archive_html .= '</div>';
	
		return $content . $archive_html;
	}
	// else, do nothing
	else {
		return $content;
	}
}

function tb_widget_control_panel() {
	$tb_o = get_option('tweet-blender');

	$cp_html = '<input type="hidden" name="tb_new_data" value="Y">';
	
	// select tweets num
	$cp_html .= '<label for="widget_tweets_num">Show <select name="widget_tweets_num">';
	for ($i = 1; $i <= 10; $i++) {
		$cp_html .= '<option value="' . $i . '"';
		if ($i == $tb_o['widget_tweets_num']) {
			$cp_html .= ' selected';
		}
		$cp_html .= '>' . $i . '</option>';
	}
	$cp_html .= '</select> tweets</label><br>';
	
	// show photos checkbox
	$cp_html .= '<label for="widget_show_photos"><input type="checkbox" name="widget_show_photos"';
	if ($tb_o['widget_show_photos']) {
		$cp_html .= " checked";
	}
	$cp_html .= '> Show user\'s photo</label><br>';
	
	// show source checkbox
	$cp_html .= '<label for="widget_show_source"><input type="checkbox" name="widget_show_source"';
	if ($tb_o['widget_show_source']) {
		$cp_html .= " checked";	
	}
	$cp_html .= '> Show tweet source (e.g. &quot;from web&quot;)</label>';
	
	echo $cp_html; 
 
    // See if the user has posted us some information
    if( $_POST['tb_new_data'] == 'Y' ) {
    	$tb_o = array(
		   	'widget_show_photos' => $_POST['widget_show_photos'],
		   	'widget_show_source'=> $_POST['widget_show_source'],
		   	'widget_tweets_num' => $_POST['widget_tweets_num']
		);
		update_option('tweet-blender',$tb_o);
  	}
}

/**
 * Create HTML for the sidebar widget
 *
 * @uses tb_get_archive_post_id()
 * @uses tb_make_tweet_list()
 */
function tb_widget($args) {

	global $post;

    echo $args['before_widget'];

	// don't show widget on the archive page
	$archive_post_id = tb_get_archive_post_id();
	if ($post->ID != $archive_post_id) {
	
		$widget_html = '<div id="tweetblender">';
		$widget_html .= '<img id="twitterlogo" src="' . plugins_url('tweet-blender/img/twitter-logo.png') . '" alt="Twitter Logo">';
		$widget_html .= '<a id="infolink" href="http://kirill-novitchenko.com/tweet-blender/" target="_blank" title="TweetBlender info"> </a>';
		$widget_html .= '<a id="refreshlink" href="javascript:TB_showLoader();TB_blend();" title="Refresh Tweets"><img src="' . plugins_url('tweet-blender/img/ajax-refresh-icon.gif') . '" alt="Refresh"></a>';
		$widget_html .= '<div id="tb_loading">Loading...</div>';
		$widget_html .= '<ul id="tweetlist" style="display:none"></ul>';
		$widget_html .= '<a id="archivelink" href="' . get_permalink($archive_post_id) . '" style="display:none;">view all &raquo;</a>';
		$widget_html .= '</div>';
		
		echo $widget_html;
	}

	echo $args['after_widget'];
}

function tb_get_archive_post_id() {
	// if we already have page id saved as option, return it
	if ($post_id = get_option('tb_archive_page_id')) {
		return $post_id;
	}
	// else if we have such a page already, get its id and store as option
	else if ($post = get_page_by_path('tweets-archive')) {
		$post_id = $post->post_name;
		update_option('tb_archive_page_id',$post_id);
		return $post_id;
	}
	// else create such a page 
	else if ($post_id = wp_insert_post(array(
			  'post_status' => 'publish',
			  'post_type' => 'page',
			  'post_author' => 1,
			  'post_title' => 'Twitter Feed',
			  'post_content' => 'Our twitter feed.',
			  'post_name' => 'tweets-archive'
		))) {
			update_option('tb_archive_page_id',$post_id);
			return $post_id;
	}
	else {
		return null;
	}
}

include_once('admin-page.php');
?>
